//---[ Current navigation helper object ]---------------------------------------
//
// This object contains the properties of the currently selected item, window
// and current folder properties. When the user browses the media library, it
// contains the library type and ID
//
$(document).ready(function(){

var zeitdummyminneu;
var zeitdummysekneu;
var zeitdummyminalt;
var zeitdummysekalt;
var videotype;
var h;
var m;
var s;

var currentNavigation = {
	//---[ Properties ]---------------------------------------------------------
	'Container.FolderName': "",
	'System.CurrentWindow': "",
	'System.CurrentControl': "",
	'Window.Property(xmlfile)': "",
	'ListItem.DBTYPE': "",
	'ListItem.DBID': 0,
	'lastCompare': false,

	//---[ Set new navigation data ]--------------------------------------------
	set: function(data) {
		this['Container.FolderName'] = data['Container.FolderName'];
		this['System.CurrentWindow'] = data['System.CurrentWindow'];
		this['System.CurrentControl'] = data['System.CurrentControl'];
		this['Window.Property(xmlfile)'] = data['Window.Property(xmlfile)'];
		this['ListItem.DBTYPE'] = data['ListItem.DBTYPE'];
		this['ListItem.DBID'] = data['ListItem.DBID'];
	},

	//---[ Compare value with current property value ]--------------------------
	compare: function(property, value) {
		this.lastCompare = this[property] == value;
		return this.lastCompare;
	},

	//---[ Determine if (current) window is a fullscreen window ]---------------
	isFullScreenWindow: function(xmlfile) {
		if (!xmlfile) xmlfile = this['Window.Property(xmlfile)'];
		return (xmlfile == "VideoFullScreen.xml") || (xmlfile == "MusicVisualisation.xml") || (xmlfile == "SlideShow.xml");
	}
}

//---[ Currently playing item object ]------------------------------------------
//
// This object contains all the information about the currently playing item
// (which should not include picture items!). It also provides a method for
// calculating the local dimmer timeout. For short media, the screen will not
// dim otherwise. When (un)setting the object, viewer methods are called.
//
var currentItem = {
	//---[ Properties ]---------------------------------------------------------
	'art': { },
	'fanart': "",
	'label': "",
	'type': "",
	'plot': "",
	'streamdetails': "",
	'studio': "",
	'showtitle': "",
	'season': "",
	'episode': "",
	'file': "",
	'duration': "",
	'runtime': "",
	'displayartist': "",
	'description': "",
	'title': "",
	//'totaltime': "",

	//---[ Path to image ]------------------------------------------------------
	getImageURL: function(art) {
		var imagepath = "/image/" + encodeURI(art);
		return imagepath;
	},

	//---[ Set new item playing ]-----------------------------------------------
	set: function(data) {
		this['art'] = data['art'] || { };
		this['fanart'] = data['fanart'] || "";
		this['label'] = data['label'] || "";
		this['type'] = data['type'] || "";
		this['plot'] = data['plot'] || "";
		this['streamdetails'] = data['streamdetails'] || "";
		this['studio'] = data['studio'] || "";
		this['showtitle'] = data['showtitle'] || "";
		this['season'] = data['season'] || "";
		this['episode'] = data['episode'] || "";
		this['file'] = data['file'] || "";
		this['duration'] = data['duration'] || "";
		this['runtime'] = data['runtime']  || "";
		this['displayartist'] = data['displayartist']  || "";
		this['description'] = data['description']  || "";
		this['title'] = data['title']  || "";
		//this['totaltime'] = data['totaltime']  || "";

		//---[ Show item details ]----------------------------------------------
		// Background fanart
		var fanart = this['fanart'] ? this.getImageURL(this['fanart']) : smallScreenView.defaultBackground[this.type];

		// item art
		var art;
		if (this.type == "episode") {
			art = this.art["tvshow.poster"];
		} else if (this.type == "movie") {
			art = this.art["poster"];
		} else if ((this.type == "song") && this.art['album.thumb']) {
			art = this.art["album.thumb"];
		} else {
			art = this.art['thumb'];
		}
		if (!art) art = smallScreenView.defaultArt[this.type];
		else art = this.getImageURL(art);

		// Item details
		var detailTitle;
		var detailSubtitle;
		var detailText;
		// Type = TVShow episode
		if (this.type == "episode") {
			detailTitle = this.showtitle;
			detailSubtitle = "Episode " + this.season + "x" + this.episode + ": " + this.label;
			detailText = this.plot || "";
		// Details on audio song
		} else if (this.type == "song") {
			detailTitle = this.label;
			detailSubtitle = this.displayartist;
			detailText = this.description || "";
		// Details on movie
		} else if (this.type == "movie") {
			detailTitle = this.label;
			detailSubtitle = this.studio.join(', ');
			detailText = this.plot || "";
		// Details on channel
		} else if (this.type == "channel") {
			detailTitle = this.title;
			detailSubtitle = "";
			detailText = this.plot || "";
		// Some unknown item (for example, the YouTube plugin)
		} else if (this.type == "unknown") {
			detailTitle = this.label;
			detailSubtitle = "";
			detailText = this.plot.replace(/\[CR]/g, "\n").replace(/\[([\/a-zA-Z])+]/g, "");
		// Details on other items
		} else {
			detailTitle = this.label;
			detailSubtitle = "";
			detailText = this.plot || "";
		}

		// Fanart as background and show details
		smallScreenView.setBackground(fanart);
		smallScreenView.setItemDetails(this.type, art, detailTitle, detailSubtitle, detailText);
	},

	//---[ Clear item playing ]-------------------------------------------------
	unset: function() {
		this['art'] = { };					// Clear all properties
		this['fanart'] = "";
		this['label'] = "";
		this['type'] = "";
		this['plot'] = "";
		this['streamdetails'] = "";
		this['studio'] = "";
		this['showtitle'] = "";
		this['season'] = "";
		this['episode'] = "";
		this['file'] = "";
		this['duration'] = "";
		this['runtime'] = "";
		this['displayartist'] = "";
		this['description'] = "";
		this['title'] = "";
		//this['totaltime'] = "";

		smallScreenView.setRuntime();					// No runtime
		smallScreenView.unsetItemDetails();				// Clear item art and info
		if (!currentNavigation.isFullScreenWindow()) {	// Restore background to skin if no media is playing fullsceeen
			smallScreenView.setBackground();
		}
	},

	//---[ Determine what length the take for local dimming ]-------------------
	minScreensaverTime: function() {
		if (this['duration']) {
			return (this['duration'] / 10) * 1000;
		} else if (this['runtime']) {
			return (this['runtime'] / 10) * 1000;
		} else {
			return Number.MAX_VALUE;
		}
	}
}

//---[ Viewer object ]----------------------------------------------------------
//
// This object handles showing and hiding information, images and background.
// It also handles the local dimmer timeout and fading
//
var smallScreenView = {
	//---[ Properties ]---------------------------------------------------------
	screensaverTime: null,				// Number of miliseconds before screensaver kicks in
	screensaverTimer: null,				// Timer to our local screensaver to be fired when playback kicks in
	fadeDuration: 500,					// FadeIn, FadeOut, FadeTo duration
	currentBackgroundURL: null,			// Last loaded background URL

	// Default background (also while playing media)
	defaultBackground: {
		skin: "images/skindefault.jpg",
		song: "images/DefaultSong.jpg",
		episode: "images/DefaultShow.jpg",
		movie: "images/DefaultMovie.jpg"
	},

	// Default item art
	defaultArt: {
		song: "images/DefaultAlbumCover.png",
		movie: "images/DefaultVideo.png"
	},

	//---[ Set background ]-----------------------------------------------------
	setBackground: function(url) {
		if ($("#fanart").is(':animated')) {
			$("#fanart").stop(true, true);
			$("#fanart").hide();
		}
		if (!url) url = this.defaultBackground.skin;
		if (url != this.currentBackgroundURL) {
			this.currentBackgroundURL = url;
			var preloadImage = new Image();
			preloadImage.src = url;
			$(preloadImage).load(function() {
				$("#fanart").css("background-image", "url(" + url + ")");
				$("#fanart").fadeIn(0, function() {
					$("#container").css("background-image", $("#fanart").css("background-image"));
					$("#fanart").hide();
				});
			});
		}
	},

	//---[ Remove item art and details ]----------------------------------------
	unsetItemDetails: function() {
		$("#details").scrollTop(0);
		if ($("#art").is(':animated')) {
			$("#art").stop(true, true);
		}
		if ($("#details").is(':animated')) {
			$("#details").stop(true, true);
		}
		$("#art, #details").fadeOut(0);
	},

	//---[ Set item art and details ]--------------------------------------------
	setItemDetails: function(type, art, detailTitle, detailSubtitle, detailText) {
		$("#details").scrollTop(0);
		if ($("#art").is(':animated')) {
			$("#art").stop(true, true);
		}
		if ($("#details").is(':animated')) {
			$("#details").stop(true, true);
		}
		if ($("#art").is(":visible")) {
			$("#art, #details").fadeOut(0, function() {
				smallScreenView.setItemDetails(type, art, detailTitle, detailSubtitle, detailText);
			});
		} else {
			$("#details h1").text(detailTitle);
			if (detailTitle.length > 20) {
				$("#details h1").html('<marquee scrollamount="15">' + detailTitle +'</marquee>');
			}
			$("#details h2").text(detailSubtitle);
			$("#details p").text(detailText);
			var preloadImage = new Image();
			$(preloadImage).error(function() {
				preloadImage.src = (type == "song") ? this.defaultArt.song : this.defaultArt.movie;
			});
			$(preloadImage).load(function() {
				$("#art").attr('src', preloadImage.src);
				$("#art, #details").fadeIn(0);
				$("#details").perfectScrollbar('update')
			});
			preloadImage.src = art;
		}
	},

	//---[ Set local screensaver timer on/off ]---------------------------------
	setScreensaverTimer: function(on, screensaverTime) {
		// If a timer is already running, cancel it
		if (this.screensaverTimer) {
			clearTimeout(this.screensaverTimer);
			this.screensaverTimer = null;
		}
		if (on) {
			// If no specific time has been given, set to default time which corresponds to Kodi screensaver time
			if (!screensaverTime) screensaverTime = this.screensaverTime;
			// And set the new timer for the screensaver if we have a duration
			if (videotype != "channel") {
			if (screensaverTime > 0) {
				this.screensaverTimer = setTimeout(function() {
					smallScreenView.setScreensaver(true);
				},
				screensaverTime);
			}
			}
		}
	},

	//---[ Switch local screensaver on / off ]----------------------------------
	setScreensaver: function(on) {
		if (on) {
			$("#fanart").fadeTo(this.fadeDuration, 0);
			$("#fanart").hide();
			$("#item").fadeTo(this.fadeDuration, 0);
			$("#item").hide();
			$("#foot").fadeTo(this.fadeDuration, 0);
			$("#foot").hide();
			$("#container").css("background-image", "none");
			$("#container").css("background-color", "black");
			$(".containerclk").show();
			$(".containerclk").fadeTo(this.fadeDuration, 1);
			scrsvr();
		} else {
			$(".containerclk").fadeTo(this.fadeDuration, 0);
			$(".containerclk").hide();
			$("#container").css("background-color", "transparet");
			$("#container").css("background-image", "url(../images/skindefault.jpg)");
			$("#fanart").fadeTo(this.fadeDuration, 1);
			$("#fanart").show();
			$("#item").fadeTo(this.fadeDuration, 1);
			$("#item").show();
			$("#foot").fadeTo(this.fadeDuration, 1);
			$("#foot").show();
			this.setScreensaverTimer(false);
		}
	},

	//---[ Set current window (optional folder) ]-------------------------------
	setCurrentWindowName: function(currentWindow, folderName) {
		var windowName = currentWindow;
		if (folderName) windowName += "/" + folderName;
		//scrsvr();
		//$("#windowname").text(h + ":" + m);
	},

	//---[ Set current control ]------------------------------------------------
	setCurrentControl: function(label) {
		if (!label) {
			$("#control").fadeOut(this.fadeDuration);
		} else {
			$("#control").text(label);
			if (!$("#control").is(":visible")) {
				$("#control").fadeIn(this.fadeDuration);
			}
		}
	},

	//---[ Set runtime ]--------------------------------------------------------
	setRuntime: function(time, totaltime) {
		var elapsed;
		var total;
		if (time) {
			elapsed = time.hours + ':' + ('00' + time.minutes).slice(-2) + ':' + ('00' + time.seconds).slice(-2);
			total = totaltime.hours + ':' + ('00' + totaltime.minutes).slice(-2) + ':' + ('00' + totaltime.seconds).slice(-2);
		} else {
			elapsed = "0:00:00";
			total = "0:00:00";
		}
		$("#elapsed").text(elapsed);
		$("#total").text(total);
	},

	//---[ Show notification ]--------------------------------------------------
	showNotification: function(message) {
		if (message) {
			if ($("#notification").is(':animated')) {
				$("#notification").stop(true, true);
			}
			$("#notification").text(message);
			$("#notification").show();
			$("#notification").delay(this.fadeDuration * 2).fadeOut(this.fadeDuration);
		}
	},

	//---[ Set error message ]--------------------------------------------------
	setErrorMessage: function(message) {
		if (message) {
			$("#showerror").text(message);
			$("#showerror").css('display', "flex");
		} else {
			$("#showerror").css('display', "none");
			$("#showerror").text('');
		}
	}
}

//---[ Websocket client object ]------------------------------------------------
//
// This object handles the connection and data send and received over the
// websocket connection and reacts to incoming messages and events
//
var webclient = {
	//---[ Properties ]---------------------------------------------------------
	ws: null,							// Pointer to web socket
	player: null,						// Pointer to player
	speed: -1,							// Playing speed (0 = paused, 1 = normal)
	timer: null,						// Pointer to loop timer
	interval: 250,						// Tick interval (in miliseconds) for timer

	//---[ Constants ]----------------------------------------------------------
	'KEY': {
		'LEFT': 37, 'UP': 38, 'RIGHT': 39, 'DOWN': 40, 'ESC': 27, 'BACKSPACE': 8, 'ENTER': 13 , 'SPACE': 32, 'X': 88,
		'NUMPLUS': 107, 'NUMMIN': 109, 'MIN': 189,  'EQUAL': 187
	},

	//---[ Open websocket ]-----------------------------------------------------
	open: function() {
		this.ws = new WebSocket('ws://' + window.location.hostname + ':9090/jsonrpc');
		if (this.ws) {
			this.ws.onclose = this.wsclose;
			this.ws.onmessage = this.wsmessage;
			this.ws.onerror = this.wserror;
			this.ws.onopen = this.wsopen;
		} else {
			alert("Unable to open the websocket.");
		}
	},

	//---[ Handle websocket opened event ]--------------------------------------
	wsopen: function(event) {
		// Get the screensaver timeout from Kodi
		webclient.send(
			"Settings.GetSettingValue",
			{ "setting": "screensaver.time" },
			"screensaver.time"
		);
		// Deactivate screensavers
		smallScreenView.setScreensaver(false);
		webclient.KodiScreensaverOff();
		// Get active players, Kodi might already be playing
		webclient.send("Player.GetActivePlayers");
		// Start the loop timer
		webclient.timer = window.setInterval(webclient.handleInterval, webclient.interval);
	},

	//---[ Handle websocket error event ]---------------------------------------
	wserror: function(event) {
		// Log error to console
		console.log(event);
		// Stop the loop timer
		clearTimeout(webclient.timer);		// Restart the looptimer
		webclient.timer = window.setInterval(webclient.handleInterval, webclient.interval);
	},

	//---[ Handle websocket closed event ]--------------------------------------
	wsclose: function(event) {
		console.log(event);					// Log reason of closing
		clearTimeout(webclient.timer);		// Disable the timer
		//smallScreenView.setErrorMessage('Connection to Kodi has been closed. Click here to open the connection again.');
		webclient.open();
	},

	//---[ Handle incoming messages event ]-------------------------------------
	wsmessage: function(event) {
		try {
			var data = JSON.parse(event.data);
		} catch (error) {
			console.log(error);
		}

		if (data) {
			// Message is a response to a request
			if (data.id) {
				webclient.handleMessage(data.id, data);
			// Message is a notification
			} else {
				webclient.handleMessage(data.method, data);
			}
		}
	},

	//---[ Handle loop timer interval ]-----------------------------------------
	handleInterval: function() {
		// If there is a active player and not known to be paused, get the properties
		if (webclient.player && (webclient.speed != 0)) {
			webclient.send("Player.GetProperties", {
				"properties": ["type", "speed", "time", "percentage", "totaltime"],
				"playerid": webclient.player.playerid,
			});
		}

		// Get the current navigation properties
		webclient.send(
			"XBMC.GetInfoLabels",
			{ "labels": [
				"Container.FolderName", "System.CurrentWindow", "System.CurrentControl", "Window.Property(xmlfile)",
				"ListItem.DBTYPE", "ListItem.DBID"
			] },
			'CurrentNavigation'
		);
	},

	//---[ Handle messages from websocket (notifications and responses) --------
	handleMessage: function(method, data) {
		switch(method) {
			// Response to get player properties (current time and total time)
			case "Player.GetProperties":
				if (data.result) {
					//zeitdummymin = data.result.totaltime.minutes;
					//zeitdummysek = data.result.totaltime.seconds;
					this.speed = data.result.speed;
					videotype = currentItem.type;
					smallScreenView.setRuntime(data.result.time, data.result.totaltime);
				}
				if (zeitdummyminalt === "") {
					zeitdummyminalt = data.result.totaltime.minutes;
				}
				if (zeitdummysekalt === "") {
					zeitdummysekalt = data.result.totaltime.seconds;
				}
				zeitdummyminneu = data.result.totaltime.minutes;
				zeitdummysekneu = data.result.totaltime.seconds;
				if (zeitdummyminalt != zeitdummyminneu || zeitdummysekalt != zeitdummysekneu) {
					this.send("Player.GetActivePlayers");
					zeitdummyminalt = zeitdummyminneu;
					zeitdummysekalt = zeitdummysekneu;
				}
				break;

			// Response to get navigation properties
			case "CurrentNavigation":
				if (data.result) {
					// Do not handle navigation when item is playing fullscreen (no item updates)
					if (!currentNavigation.isFullScreenWindow(data.result['Window.Property(xmlfile)'])) {
						this.handleCurrentNavigation(data.result)
					// Media playing, but helper object not propagated yet, so we probably need to set the local screensaver too
					} else if (currentNavigation['Window.Property(xmlfile)'] != data.result['Window.Property(xmlfile)']) {
						smallScreenView.setCurrentControl();
						smallScreenView.setCurrentWindowName(data.result['System.CurrentWindow']);
						currentNavigation.set(data.result);
						smallScreenView.setScreensaverTimer(true, Math.min(currentItem.minScreensaverTime(), smallScreenView.screensaverTime));
					}
				}
				break

			// Notification that player has started
			case "Player.OnPlay":
				if (data.params.data.item && data.params.data.item.type != "picture") {
					if (this.speed != 0) {
						this.send("Player.GetActivePlayers");	// Request the active players only when not paused
					}
					this.speed = 1;		// Initialize speed on normal play speed
					if (currentNavigation.isFullScreenWindow()) {
						smallScreenView.setScreensaver(false);		// Deactivate the local screensaver
						smallScreenView.setScreensaverTimer(true, Math.min(currentItem.minScreensaverTime(), smallScreenView.screensaverTime));
					}
				}
				break;

			// Notification that player has stopped
			case "Player.OnStop":
				if (data.params.data.item && data.params.data.item.type != "picture") {
					this.player = null;						// No active players
					this.speed = -1;						// No speed value
					currentItem.unset();					// No current item
				}
				break;

			// Notification that player has paused or playback position has changed
			case "Player.OnPause":
				if (data.params.data.item && data.params.data.item.type != "picture") {
					smallScreenView.showNotification("Pause");
					smallScreenView.setScreensaver(false);		// Deactivate the local screensaver
				}
				break;

			case "Player.OnSeek":
				smallScreenView.setScreensaver(false);		// Deactivate the local screensaver
				smallScreenView.setScreensaverTimer(true, Math.min(currentItem.minScreensaverTime(), smallScreenView.screensaverTime));
				if (data.params.data) {
					var seconds = (data.params.data.player.seekoffset.hours * 3600) + (data.params.data.player.seekoffset.minutes * 60) + data.params.data.player.seekoffset.seconds;
					var seeking = (seconds < 0) ? "-" : "+";
					if (data.params.data.player.seekoffset.hours != 0) seeking += Math.abs(data.params.data.player.seekoffset.hours) + ':';
					seeking += ('00' + Math.abs(data.params.data.player.seekoffset.minutes)).slice(-2) + ':';
					seeking += ('00' + Math.abs(data.params.data.player.seekoffset.seconds)).slice(-2);
					smallScreenView.showNotification("Seeking: " + seeking);
				}
				break;

			// Notification that Kodi will quit
			case "System.OnQuit":
				this.ws.close();						// Close websocket
				break;

			// Response to get active players (needs work - we can get more than one player!)
			case "Player.GetActivePlayers":
				var result = (data.result && data.result.length) ? data.result.shift() : null;
				while (result && (result.type == "picture") && (data.result.length > 0)) {
					result = data.result.shift();
				}
				if (result && (result.type != "picture")) {
					this.player = result;					// Save active player to property
					this.send("Player.GetItem", {
						"properties": ["art", "fanart", "plot", "streamdetails", "studio", "showtitle", "season", "episode", "file", "duration", "runtime", "displayartist", "description", "title"],
						"playerid": this.player.playerid,
					});
				}
				break;

			// Response to get the current item playing
			case "Player.GetItem":
				if (data.result && data.result.item) {
					currentItem.set(data.result.item);
				}
				break;

			// Response to get the Movie library data
			case "VideoLibrary.GetMovieDetails":
				if (data.result) {
					// Background (fan)art
					var fanart = this.getImageURL('fanart', 'movie', data.result.moviedetails.art);
					if (!fanart) fanart = smallScreenView.defaultBackground['movie'];
					smallScreenView.setBackground(fanart);
				}
				break;

			// Response to get the Movie set library data
			case "VideoLibrary.GetMovieSetDetails":
				if (data.result) {
					// Background (fan)art
					var fanart = this.getImageURL('fanart', 'movie', data.result.setdetails.art);
					if (!fanart) fanart = smallScreenView.defaultBackground['movie'];
					smallScreenView.setBackground(fanart);
				}
				break;

			// Response to get the TVShow library data
			case "VideoLibrary.GetTVShowDetails":
				if (data.result) {
					// Background (fan)art
					var fanart = this.getImageURL('fanart', 'tvshow', data.result.tvshowdetails.art);
					if (!fanart) fanart = smallScreenView.defaultBackground['episode'];
					smallScreenView.setBackground(fanart);
				}
				break;

			// Response to get the TVShow's season library data
			case "VideoLibrary.GetSeasonDetails":
				if (data.result) {
					// Background (fan)art
					var fanart = this.getImageURL('fanart', 'season', data.result.seasondetails.art);
					if (!fanart) fanart = smallScreenView.defaultBackground['episode'];
					smallScreenView.setBackground(fanart);
				}
				break;

			// Response to get the TVShow's episode library data
			case "VideoLibrary.GetEpisodeDetails":
				if (data.result) {
					// Background (fan)art
					var fanart = this.getImageURL('fanart', 'episode', data.result.episodedetails.art);
					if (!fanart) fanart = smallScreenView.defaultBackground['episode'];
					smallScreenView.setBackground(fanart);
				}
				break;

			// New item selected for playing
			case "Playlist.OnClear":
				if (this.speed == 0) this.speed = -1;	// Reset speed if item was paused, forcing item refresh with OnPlay
				break;

			// Kodi screensaver activated
			case "GUI.OnScreensaverActivated":
				smallScreenView.setScreensaver(true);
				break;

			// Kodi screensaver deactivated
			case "GUI.OnScreensaverDeactivated":
				smallScreenView.setScreensaver(false);
				break;

			// Kodi setting: screensaver timeout for local screensaver timeout. For now, always set a default timeout
			case "screensaver.time":
				smallScreenView.screensaverTime = (data.result && data.result.value) ? data.result.value * 60000 : 120000;
				break;

			// Volume changed
			case "Application.OnVolumeChanged":
				if (data.params.data) {
					if (data.params.data.muted) smallScreenView.showNotification("Mute");
					else smallScreenView.showNotification("Volume: " + Math.round(data.params.data.volume) + "%");
				}
				break;

			// Unmonitored messages
			default:
				//console.log("Unmonitored message:");
				//console.log(data);
		}
	},

	//---[ Handle navigation messages (user navigates Kodi) ]-------------------
	handleCurrentNavigation: function(data) {
		// Window updated
		if (!currentNavigation.compare('Window.Property(xmlfile)', data['Window.Property(xmlfile)'])) {
			smallScreenView.setCurrentWindowName(data['System.CurrentWindow'], data['Container.FolderName']);
			smallScreenView.setCurrentControl(data['System.CurrentControl']);
		// Folder updated
		} else if (!currentNavigation.compare('Container.FolderName', data['Container.FolderName'])) {
			smallScreenView.setCurrentWindowName(data['System.CurrentWindow'], data['Container.FolderName']);
			smallScreenView.setCurrentControl(data['System.CurrentControl']);
		// Control updated
		} else if (!currentNavigation.compare('System.CurrentControl', data['System.CurrentControl'])) {
			smallScreenView.setCurrentControl(data['System.CurrentControl']);
		}

		// If an update has taken place...
		if (!currentNavigation.lastCompare) {
			// Unset currently running screensaver(timer) if any
			if (smallScreenView.screensaverTimer) smallScreenView.setScreensaver(false);

			// Only show fanart when there is no item playing)
			if (!this.player) {
				// User was browsing Library items, but not anymore
				if (data['ListItem.DBTYPE'] == "") {
					smallScreenView.setBackground();		// Restore regular background
	
				// User was browsing Season library item, but it's a "virtual" item with no properties (please fix this, Kodi guys. :-/ )
				} else if ((data['ListItem.DBTYPE'] == "season") && (data['System.CurrentControl'].substr(0, 2) == "[*")) {
					smallScreenView.setBackground();		// Restore regular background
	
				// User is browsing a library item 'movie'
				} else if (data['ListItem.DBTYPE'] == "movie") {
					webclient.send("VideoLibrary.GetMovieDetails", {
						"properties": ["art"],
						"movieid": parseInt(data['ListItem.DBID'], 10)
					});
	
				// User is browsing a library item 'set' (of movies)
				} else if (data['ListItem.DBTYPE'] == "set") {
					webclient.send("VideoLibrary.GetMovieSetDetails", {
						"properties": ["art"],
						"setid": parseInt(data['ListItem.DBID'], 10)
					});
	
				// User is browsing a library item 'tvshow'
				} else if (data['ListItem.DBTYPE'] == "tvshow") {
					webclient.send("VideoLibrary.GetTVShowDetails", {
						"properties": ["art"],
						"tvshowid": parseInt(data['ListItem.DBID'], 10)
					});
	
				// User is browsing a library item 'season'
				} else if (data['ListItem.DBTYPE'] == "season") {
					webclient.send("VideoLibrary.GetSeasonDetails", {
						"properties": ["art", "tvshowid"],
						"seasonid": parseInt(data['ListItem.DBID'], 10)
					});
	
				// User is browsing a library item 'episode'
				} else if (data['ListItem.DBTYPE'] == "episode") {
					webclient.send("VideoLibrary.GetEpisodeDetails", {
						"properties": ["art", "tvshowid"],
						"episodeid": parseInt(data['ListItem.DBID'], 10)
					});
				}
			}

			// Populate currentNavigation helper object
			currentNavigation.set(data);
		}
	},

	//---[ Determine art path ]-------------------------------------------------
	getImageURL: function(type, itemType, art) {
		var imagepath = null;
		if (type && itemType && art) {
			// Requested image is episode tvshow fanart (background)
			if ((type == "fanart") && (itemType == "episode")) {
				imagepath = art["tvshow.fanart"];
			// Requested image is season tvshow fanart (background)
			} else if ((type == "fanart") && (itemType == "season")) {
				if (art['fanart']) imagepath = art['fanart'];
				else imagepath = art["tvshow.fanart"];
			// Requested image is music artist fanart (background)
			} else if ((type == "fanart") && (itemType == "song")) {
				imagepath = art["artist.fanart"];
			// All other fanart (background)
			} else if (type == "fanart") {
				imagepath = art.fanart;

			// Requested image is episode tvshow poster art
			} else if ((type == "art") && (itemType == "episode")) {
				imagepath = art["tvshow.poster"];
			// Requested image is movie poster
			} else if ((type == "art") && (itemType == "movie")) {
				imagepath = art.poster;
			// Requested image is album art
			} else if ((type == "art") && (itemType == "song")) {
				if (art["album.thumb"]) {
					imagepath = art["album.thumb"];
				} else {
					imagepath = art.thumb;
				}
			// Any other thumb
			} else if (type == "art") {
				imagepath = art.thumb;
			}

			// Find out the URL
			if (imagepath) {
				imagepath = "/image/" + encodeURI(imagepath);
			}
		}
		return imagepath;
	},

	//---[ Set Kodi screensaver off ]-------------------------------------------
	KodiScreensaverOff: function() {
		webclient.send("Input.ExecuteAction", {
			"action": "noop",
		});
	},

	//---[ Send input through to Kodi ]-----------------------------------------
	sendInput: function(key) {
		switch(key) {
			case this.KEY.LEFT:
				if (currentNavigation.isFullScreenWindow()) {
					webclient.send("Input.ExecuteAction", {
						"action": "stepback",
					});
				} else {
					webclient.send("Input.ExecuteAction", {
						"action": "left",
					});
				}
				break;
			case this.KEY.RIGHT:
				if (currentNavigation.isFullScreenWindow()) {
					webclient.send("Input.ExecuteAction", {
						"action": "stepforward",
					});
				} else {
					webclient.send("Input.ExecuteAction", {
						"action": "right",
					});
				}
				break;
			case this.KEY.UP:
				if (currentNavigation.isFullScreenWindow()) {
					webclient.send("Input.ExecuteAction", {
						"action": "bigstepforward",
					});
				} else {
					webclient.send("Input.ExecuteAction", {
						"action": "up",
					});
				}
				break;
			case this.KEY.DOWN:
				if (currentNavigation.isFullScreenWindow()) {
					webclient.send("Input.ExecuteAction", {
						"action": "bigstepback",
					});
				} else {
					webclient.send("Input.ExecuteAction", {
						"action": "down",
					});
				}
				break;
			case this.KEY.ESC:
				webclient.send("Input.ExecuteAction", {
					"action": "parentdir",
				});
				break;
			case this.KEY.BACKSPACE:
				webclient.send("Input.ExecuteAction", {
					"action": "back",
				});
				break;
			case this.KEY.ENTER:
				webclient.send("Input.ExecuteAction", {
					"action": "select",
				});
				break;
			case this.KEY.SPACE:
				webclient.send("Input.ExecuteAction", {
					"action": "pause",
				});
				break;
			case this.KEY.X:
				webclient.send("Input.ExecuteAction", {
					"action": "stop",
				});
				break;
			case this.KEY.NUMPLUS:
			case this.KEY.EQUAL:
				webclient.send("Input.ExecuteAction", {
					"action": "volumeup",
				});
				break;
			case this.KEY.NUMMIN:
			case this.KEY.MIN:
				webclient.send("Input.ExecuteAction", {
					"action": "volumedown",
				});
				break;
			default:
				//console.log(key);
		}
	},

	//---[ Send messages over websocket ]---------------------------------------
	send: function(method, params, id) {
		if (!id) id = method;
		var msg = {
			"jsonrpc": "2.0", 
			"method": method, 
			"id": id
		};
		if (params) {
			msg.params = params;
		}
		if (this.ws.readyState == 1) {
			this.ws.send(JSON.stringify(msg));
		}
	}
}

function scrsvr(){
startTime();
apiJson();
setInterval(apiJson, 900000);
//moveDiv();
setInterval(moveDiv, 20000);
}

	function startTime() {
		var today = new Date();
		h = today.getHours();
		m = today.getMinutes();
		s = today.getSeconds();
//		var ampm = h >= 12 ? 'PM' : 'AM'; // Display AM or PM
		h = checkTime(h);
		m = checkTime(m);
		s = checkTime(s);
		//Convert to 12-hour (non-military time)
//		if (h > 12) {
//			h -= 12;
//		} else if (h === 0) {
//			h = 12;
//		};
		document.getElementById('clock').innerHTML =
		h + ":" + m + ":" + s + " ";
		var t = setTimeout(startTime, 500);
		var todayDate = new Date();
		var dd = todayDate.getDate();
		var mm = todayDate.getMonth()+1; //January is 0!
		var yyyy = todayDate.getFullYear();
		if(dd<10) {
			dd = '0' + dd
		}
		if(mm<10) {
			mm = '0' + mm
		};
		document.getElementById('whatDate').innerHTML = dd + '.' + mm + '.' + yyyy;
	}

	function checkTime(i) {
		if (i < 10) {i = "0" + i};  // add zero in front of numbers < 10
		return i;
	}

	function moveDiv() {
var $span = $("#clockArea");

$span.fadeOut(1000, function() {
    var maxLeft = $(window).width() - $span.width();
    var maxTop = $(window).height() - $span.height();
    var leftPos = Math.floor(Math.random() * (maxLeft + 1))
    var topPos = Math.floor(Math.random() * (maxTop + 1))

    $span.css({ left: leftPos, top: topPos }).fadeIn(1000);
});
}

function apiJson(){
var weather = "https://api.openweathermap.org/data/2.5/weather?lat=XX.XX&lon=XX.XX&units=metric&appid=XXXX";

$.ajax({
  url: weather,
  dataType: 'jsonp',
  success: function(data){
     //var img = data.current_observation.icon;
     var tempC = data.main.temp;
     //$('#img').attr('src', "png/64x64/" + img + ".png");
     $('#temp').text(tempC + " C");
    } 
  });
}




function zeit(){
$("#windowname").text(h + ":" + m);
}

zeit();
setInterval(zeit, 500);

//---[ Main line - start the kodi webclient after everything has loaded ]-------
$(window).load(function() {
	$('#details').dragScroll({});
	$('#details').perfectScrollbar({});
	$('#container').on('click', function(event) {
		smallScreenView.setScreensaver(false);
		if (currentNavigation.isFullScreenWindow()) {
			smallScreenView.setScreensaverTimer(true);
		}
	});
	scrsvr();
	$('#showerror').on('click', function(event) {
		$(this).css('display', "none");
		webclient.open();
	});
	$(window).keydown(function(event) {
		if ((event.which == 13) || (event.which == 8)) {
			event.preventDefault();
		}
		webclient.sendInput(event.which);
	});
	//Left = 37, Up = 38, Right = 39, Down = 40,
	// ESC = 27, Enter = 13, Backspace = 8; Plus = 107, min = 109, IS-teken = 187, min-teken: 189
	webclient.open();
});
}); 